# -*- coding: utf-8 -*-
import os
import threading

import yaml
from Pyautomators.__main__ import main

'''
@author: Kaue Bonfim
'''
"""Importando bibliotecas externas"""
"""Importando bibliotecas internas"""


class GenericModel():

    def __init__(self, dict_yaml):
        self.dict_yaml = dict_yaml
        self.exec_options = []

        for option in self.dict_yaml:
            if option == 'Name' or option == 'name':
                self.exec_options.append("-Dname=" + self.dict_yaml[option])
            elif option == 'Tags' or option == 'tags':
                for arg in self.dict_yaml[option]:
                    tag_string = str(",").join(self.dict_yaml[option])
                self.exec_options.append("--tags=" + tag_string)
            elif option == 'Args' or option == 'args':
                for arg in self.dict_yaml[option]:
                    self.exec_options.append(
                        '-D' + str(arg) + '=' + str(self.dict_yaml[option][arg]))
    
    def get_feauture(self):

        for iten in self.dict_yaml.items():
            if iten[0] in ['Feature','feature']:
                self.exec_options.append(iten[1])
                break
        
class WebModel(GenericModel):

    def options_web(self, browser):
        name=self.dict_yaml['Name']
        self.exec_options.append('-Dbrowser=' + browser)

        for option in self.dict_yaml:
            if option == 'Reports' or option == 'reports':
                dir = "docs/reports/"
                self.exec_options.append('--junit')
                self.exec_options.append('--junit-directory={dir}{name}/xml/{browser}/'.format(dir=dir,name=name,browser=str(browser)))
                self.exec_options.append('--format=json')
                self.exec_options.append("-o={dir}{name}/{browser}-{name}.json".format(dir=dir,name=name,browser=str(browser)))
                self.exec_options.append('--format=json_cucumber.pretty')
                self.exec_options.append("-o={dir}cucumber/{browser}-{name}-cucumber.json".format(dir=dir,name=name,browser=str(browser)))

            elif option == "Spec" or option == 'spec':
                dir = "docs/spec/"
                self.exec_options.append('--format=sphinx.steps')
                self.exec_options.append('-o={dir}{browser}-{name}/'.format(dir=dir,browser=browser,name=name))
                
        self.get_feauture()

        return self.exec_options


class MobileModel(GenericModel):

    def options_mobile(self, device):
        name=self.dict_yaml['Name']
        self.exec_options.append('-Ddevice=' + device)        
        for option in self.dict_yaml:
            if option == 'Reports' or option == 'reports':
                dir = "docs/reports/"
                self.exec_options.append('--junit')
                self.exec_options.append('--junit-directory={dir}{name}/xml/{device}/'.format(dir=dir,name=name,device=str(device)))
                self.exec_options.append('--format=json')
                self.exec_options.append("-o={dir}{name}/{device}-{name}.json".format(dir=dir,name=name,device=str(device)))
                self.exec_options.append('--format=json_cucumber.pretty')
                self.exec_options.append("-o={dir}cucumber/{device}-{name}-cucumber.json".format(dir=dir,name=name,device=str(device)))

            elif option == "Spec" or option == 'spec':
                dir = "docs/spec/"
                self.exec_options.append('--format=sphinx.steps')
                self.exec_options.append('-o={dir}{device}-{name}/'.format(dir=dir,device=device,name=name))
                
        self.get_feauture()

        return self.exec_options


class DesktopModel(GenericModel):

    def run_desktop(self):
        name=self.dict_yaml['Name']
        for option in self.dict_yaml:
            if option == 'Reports' or option == 'reports':
                dir = "docs/reports/"
                self.exec_options.append('--junit')
                self.exec_options.append('--junit-directory={dir}{name}/xml/'.format(dir=dir,name=name))
                self.exec_options.append('--format=json')
                self.exec_options.append("-o={dir}{name}/{name}.json".format(dir=dir,name=name))
                self.exec_options.append('--format=json_cucumber.pretty')
                self.exec_options.append("-o={dir}cucumber/{name}-cucumber.json".format(dir=dir,name=name))

            elif option == "Spec" or option == 'spec':
                dir = "docs/spec/"
                self.exec_options.append('--format=sphinx.steps')
                self.exec_options.append('-o={dir}{name}/'.format(dir=dir,name=name))
                
        self.get_feauture()

        return self.exec_options

class ThreadRun(threading.Thread):
    def __init__(self, options, sub_option=None):
        threading.Thread.__init__(self)
        self.sub_option = sub_option
        self.options = options

class ThreadWeb(ThreadRun):
    def run(self):
        main(WebModel(self.options).options_web(self.sub_option))
        

class ThreadMobile(ThreadRun):
    def run(self):
        main(MobileModel(self.options).options_mobile(self.sub_option))


class ThreadDesktop(ThreadRun):
    def run(self):
        main(DesktopModel(self.options).run_desktop())


def orchestra(file):
    file = open("manager/{file_yaml}".format(file_yaml=file), "r")
    block_test = yaml.load(file)
    
    def prepare_execute(keys_yaml):
        thread_list=[]
        for key,value in keys_yaml.items():
            
            if key in ['Type','type']:
                
                if value in ["Web",'web']:
                    for key,value in keys_yaml.items():
                        
                        if key in ['Browsers','browsers']:
                            for browser in value:
                                thread_list.append(ThreadWeb(keys_yaml, browser))
                    break

                elif value in ["Mobile",'mobile']:
                    for key,value in keys_yaml.items():
                        if key in ['Devices','devices']:
                            for device in value:
                                thread_list.append(ThreadMobile(keys_yaml, device))
                    break
                    
                elif value in ["Desktop",'desktop']:
                    thread_list.append(ThreadDesktop(keys_yaml))
                    break

                else:
                    ERROR = """Undefined Type"""
                    raise Exception(ERROR)
        return thread_list


    for test in block_test:
        
        if str(test).find("Test") != -1 or str(test).find("test") != -1:
            
            for block in prepare_execute(block_test[test]):
                block.start()


        else:
            ERROR = """Undefined Test"""
            Exception(ERROR)



# -*- coding: utf-8 -*-
'''
@author: KaueBonfim
'''

import os
import re


''' Este modulo trabalha com a construção do padrão de projeto Pyautomators em BDD,
    pode conter estrutural e tatico do projeto'''


def create_project(nome_projeto, diretorio=None):
    '''Este metodo abre um projeto em com a estrutura pyautomators

    parametros:
    nome_projeto(obrigatorio):precisa ser passado o nome da pasta que cera criada para o projeto, sendo ela não existente
    diretorio:em qual caminho ou diretorio vai ser criada a pasta, se não for passado vai ser criado no atual
    Exemplo:
    Criar_Projeto("novo projeto","C:/APP")
    Criar_Projeto("novo projeto")
    Criar_Projeto("novo projeto",Ambiente.path_atual())
    '''
    ##########################################################################
    #          Criando a pasta que contera o diretorio principal do projeto          #
    ##########################################################################

    os.mkdir(nome_projeto)

    os.chdir(nome_projeto)
    ##########################################################################
    #      Gerando as pastas que contera as responsabilidades do projeto             #
    ##########################################################################

    lista_pastas_principal = [
        "bin",
        "data",
        "docs",
        "driver",
        "features",
        "lib",
        'log',
        "steps",
        "manager",
        "docs/reports",
        "data/images",
        "pages",
        "pages/navigations",
        "pages/pages"]
    for lin in lista_pastas_principal:
        os.mkdir(lin)

    text = open("__init__.py", "w")
    text.close()

    text = open("manager/__init__.py", "w")
    text.close()
    ##########################################################################
    #            Escrevendo a estrutura do arquivo environment.py                    #
    ##########################################################################
    text = open("environment.py", "w")
    text.writelines(
        '"""\tPyautomators an Framework to Test \n\n\t\t\tProject: {}"""\n\n'.format(nome_projeto))
    text.writelines(
        'from Pyautomators import fixture\nfrom Pyautomators.contrib.scenario_autoretry import scenario_retry\n')
    text.writelines("def before_all(context):\n\tpass\n\n")
    text.writelines("def before_feature(context,feature):\n\tpass\n\n")
    text.writelines("def before_scenario(context,scenario):\n\tpass\n\n")
    text.writelines("def before_tag(context,tag):\n\tpass\n\n")
    text.writelines("def after_tag(context,tag):\n\tpass\n\n")
    text.writelines("def before_step(context,step):\n\tpass\n\n")
    text.writelines("def after_step(context,step):\n\tpass\n\n")
    text.writelines("def after_scenario(context,scenario):\n\tpass\n\n")
    text.writelines("def after_feature(context,feature):\n\tpass\n\n")
    text.writelines("def after_all(context):\n\tpass\n\n")
    text.close()
    
    text = open("steps/__init__.py", "w")
    text.close()

    text = open("pages/__init__.py", "w")
    text.close()

    text = open("pages/navigations/__init__.py", "w")
    text.close()

    text = open("pages/pages/__init__.py", "w")
    text.close()



import logging
import threading
import time
from collections import deque
import traceback
class LoggerEnv():

    def __init__(self,name,level=logging.INFO,file_name='log/Test-{name}-{day}-{month}-{year}.log',format='%(asctime)s %(threadName)s :%(levelname)s:    %(message)s',datefmt='%d-%m-%y %H:%M:%S'):
        line=time.localtime()
        line=list(line)
        lis=deque()
        for h in line:
            lis.appendleft(int(h))
            if(line.index(h) == 2):
                break
        day=lis[0]
        month=lis[1]
        year=lis[2]
        self.name=name
        self.thread=threading.currentThread().getName()
        filename=file_name.format(thread=self.thread,name=self.name,day=day,month=month,year=year)
        logging.basicConfig(level=level,filename=filename,format=format,datefmt=datefmt)

    def capture_stage(self,function,breaks=True,simple=True):
        
        def capture(*args, **kwargs):
            try:
                function(*args, **kwargs)
                logging.info("...Stage {func}:Sucess!".format(func=function.__name__))
            except Exception as e :
                
                if simple:
                    logging.error("...Stage {func}:   {msg}".format(msg=str(e.args[0]),func=function.__name__))
                else:
                    trace=traceback.format_exc()
                    logging.error("...Stage {func}:\n{trace}".format(trace=trace,func=function.__name__))
                if breaks:
                    Error="\n"+str(e)
                    raise Exception(Error)
            
        return capture

    def capture_after_step(self,function,breaks=True,simple=True):
        
        def capture(*args, **kwargs):
            step=args[1]
            if 'passed' in str(step.status):
                logging.info("...Step Sucess:{key} {step}".format(key=step.keyword,step=step.name))
            else:
                logging.error("...Step {key} {step}:{status}:   {msg},filename:{path}...{line}".format(key=step.keyword,status=step.status,step=step.name,msg=step.exception,path=str(step.filename),line=str(step.line)))
            
            try:
                function(*args, **kwargs)
                logging.info("...Stage {func}:Sucess".format(func=function.__name__))
            except Exception as e :
                
                if simple:
                    logging.error("...Stage {func}:   {msg}".format(msg=str(e.args[0]),func=function.__name__))
                else:
                    trace=traceback.format_exc()
                    logging.error("...Stage {func}:\n{trace}".format(trace=trace,func=function.__name__))
                if breaks:
                    Error="\n"+str(e)
                    raise Exception(Error)
            
        return capture



# -*- coding: utf-8 -*-
import argparse

from orcautomators import created_dir, exec_file

'''
@author: Kaue Bonfim
'''
"""Importando bibliotecas externas"""
"""Importando bibliotecas internas"""
# - annotation:
# mainTitle: Pyautomators inLine
# text_description:
# - O modulo __main__ trabalha com a execução dos testes em linha de comando
# parameters:
# - string: "comando [CHAVE ARGUMENTOS]" 
# ex:
# - python: "python -m Pyautomators COMANDO [CHAVE ARGUMENTOS]"
# - python: "python -m Pyautomators criar_projeto -n Testes_Web\n"

"""So vai usar a main, caso este arquivo ou modulo seja chamado"""
if '__main__' == __name__:

    ARG = argparse.ArgumentParser()
    """criando um objeto para receber os argumentos"""

# - annotation:
# title:
###        - "**Comandos**\n"
# text_description:
###        - "**criar_projeto:** Usado para gerar projeto de automação e a arquitetura de diretório\n"
###        - "\t**CHAVES:** -f -d (Opcional)\n"
###        - "**execute:** Usado para executar um projeto orquestrado com base em um yaml\n"
###        - "\t**CHAVES:** -f -d (Opcional)\n"
###        - "**criar_doc:** Usado para gerar um documento de evidencia do projeto, com base no dos arquivos de resultados da pasta docs/reports"
###        - " as imagens para cada step deve conter o nome da funcionalidade, cenario e step no nome da imagem a qual ela pertence.\n"
###        - "\t**CHAVES:** -d(Opcional)\n"
###        - "**Master_Remote:** Usado para abrir um conector master para executar testes em clients remote\n"
###        - "\t**CHAVES:** -f \n"
###        - "**Client_Remote:** Usado para abrir um servidor master para executar testes\n"
###        - "\t**CHAVES:** -f \n"
###        - "**Gerar_zip:** Usado para abrir um servidor master para executar testes\n"

# - annotation:
###    title: "**Parametros**\n"
# text_description:
###        - "Os Parametros ou argumentos são passados para especificar a execução de um determinado comando.\n"
###        - "**-f ou --file_yaml:** Arquivo Yaml base para execucoes em threads e containers.\n"
###        - "**-n ou --nome_projeto:** Criar um projeto com a base do Pyautomators.\n"
###        - "**-d ou --diretorio:** Para indicar um diretorio para a execução das ações.\n"
###        - "**--volume_host:** Volume do host.\n"
###        - "**--volume_container:** Volume do container.\n"
###        - "**--master_host:** IP do master.\n"
###        - "**--instancia:** Instancia do Yaml a ser executada.\n"
###        - "**-i ou --retorn_result:** Retornar resultados.\n"
# ex:
# - python: "python -m Pyautomators execute -f data/run.yaml\n"
# - python: "python -m Pyautomators criar_projeto -n Testes_Web\n"

    """Adicionando os argumento"""
    ARG.add_argument("comando", help="")
    ARG.add_argument(
        "-n",
        '--name',
        required=False,
        help="Criar um projeto com a base do Pyautomators")
    ARG.add_argument(
        "-f",
        '--file_exec',
        required=False,
        default='Pyautomators.yaml',
        help="Identifica um arquivo para executar na pasta manager")

    PROJECT = vars(ARG.parse_args())

    """Chamadas de funcao para cada argumento"""
    if PROJECT['comando'] == 'create':
        created_dir.create_project(PROJECT["name"])

    elif PROJECT['comando'] == "exec":
        exec_file.orchestra(PROJECT['file_exec'])

    else:
        ERROR = """Undefined Command"""

        raise Exception(ERROR)



# -*- coding: utf-8 -*-
from setuptools import setup,find_packages

setup(name='orcautomators',
      version='1.1',
      url='',
      license='Indra Company',
      author='Kaue Bonfim',
      author_email='koliveirab@indracompany.com',
      description='Orchestration Library for Tests',
      packages=['orcautomators',],
	install_requires=['pyyaml==3.12', 'Pyautomators', 'argparse'],
      zip_safe=True,
      test_suite='test',
      long_description=open('README.md').read())